﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace DragDropWpf
{
	/// <summary>
	/// Window1.xaml の相互作用ロジック
	/// </summary>
	public partial class Window1 : Window
	{
		Point _LastMouseDownPos;

		public Window1()
		{
			InitializeComponent();

			_DragMeLabel.MouseDown += _DragMeLabel_MouseDown;
			_DragMeLabel.MouseMove += _DragMeLabel_MouseMove;
			_DragMeLabel.GiveFeedback += _DragMeLabel_GiveFeedback;
			_DragMeLabel.QueryContinueDrag += _DragMeLabel_QueryContinueDrag;

			_DropToMeLabel.MouseMove += _DropToMeLabel_MouseMove;
			_DropToMeLabel.DragEnter += _DropToMeLabel_DragEnter;
			_DropToMeLabel.DragLeave += _DropToMeLabel_DragLeave;
			_DropToMeLabel.DragOver += _DropToMeLabel_DragOver;
			_DropToMeLabel.Drop += _DropToMeLabel_Drop;
			_DropToMeLabel.AllowDrop = true;
		}

		#region Dragging source
		void _DragMeLabel_MouseDown( object sender, MouseButtonEventArgs e )
		{
			Console.WriteLine( "(src)MouseDown( {0} )", e.GetPosition(this) );
			_LastMouseDownPos = e.GetPosition( this );
		}

		void _DragMeLabel_MouseMove( object sender, MouseEventArgs e )
		{
			Console.WriteLine( "(src)MouseMove( {0} )", e.GetPosition(this) );

			// 左クリックで一定距離ドラッグされた場合はD&Dを実行
			if( e.LeftButton == MouseButtonState.Pressed )
			{
				Point pos = e.GetPosition( this );
				if( 3 < Math.Abs(pos.X - _LastMouseDownPos.X)
					|| 3 < Math.Abs(pos.Y - _LastMouseDownPos.Y) )
				{
					// D&Dでドロップ先に送信するデータを準備
					IDataObject data = new DataObject( DataFormats.Text, ":)" );
					
					// D&Dセッションを開始。
					// ドロップ先に
					// 「_DragMeLabelからCopyまたはMoveが可能なデータを送る」
					// と通知する。
					DragDropEffects allowedEffects = DragDropEffects.Copy | DragDropEffects.Move;
					DragDropEffects result = DragDrop.DoDragDrop( _DragMeLabel,
																  data,
																  allowedEffects );

					// D&Dの結果を判定
					if( result == DragDropEffects.None )
						MessageBox.Show( "D&D failed." );
					else if( result == DragDropEffects.Move )
						_DragMeLabel.Content = "Moved.";
					else
						_DragMeLabel.Content = "Copied.";
				}
			}
		}

		void _DragMeLabel_GiveFeedback( object sender, GiveFeedbackEventArgs e )
		{
			//Console.WriteLine( "(src)GiveFeedback" );
		}

		void _DragMeLabel_QueryContinueDrag( object sender, QueryContinueDragEventArgs e )
		{
			//Console.WriteLine( "(src)QueryContinueDrag" );
		}
		#endregion

		#region Dragging destination
		void _DropToMeLabel_MouseMove( object sender, MouseEventArgs e )
		{
			Console.WriteLine( "(dest)MouseMove" );
		}

		void _DropToMeLabel_DragEnter( object sender, DragEventArgs e )
		{
			e.Effects = DecideDragAction( e );
			Console.WriteLine( "(dest)DragEnter --> {0}", e.Effects );
		}

		void _DropToMeLabel_DragLeave( object sender, DragEventArgs e )
		{
			Console.WriteLine( "(dest)DragLeave" );
		}

		void _DropToMeLabel_DragOver( object sender, DragEventArgs e )
		{
			e.Effects = DecideDragAction( e );
			Console.WriteLine( "(dest)DragOver --> {0}", e.Effects );
			e.Handled = true; // これが無いと標準のイベントハンドラがマウスカーソルを戻してしまう
		}

		void _DropToMeLabel_Drop( object sender, DragEventArgs e )
		{
			e.Effects = DecideDragAction( e );
			Console.WriteLine( "(dest)Drop --> {0}", e.Effects );

			switch( e.Effects )
			{
				case DragDropEffects.Copy:
					_DropToMeLabel.Content = e.Data.GetData( DataFormats.Text );
					break;

				case DragDropEffects.Move:
					_DropToMeLabel.Content = e.Data.GetData( DataFormats.Text );
					break;

				default:
					break;
			}
		}

		DragDropEffects DecideDragAction( DragEventArgs e )
		{
			Point pos = e.GetPosition( _DropToMeLabel );

			// 単純に _DropToMeLabel の左半分ならば Copy、右半分ならば Move とする
			if( pos.X < (_DropToMeLabel.ActualWidth / 2) )
			{
				return DragDropEffects.Copy;
			}
			else
			{
				return DragDropEffects.Move;
			}
		}
		#endregion
	}
}
